@@ -36,6 +36,7 @@ PROJ_HEAD(fahey, "Fahey")
 PROJ_HEAD(fouc, "Foucaut")
 PROJ_HEAD(fouc_s, "Foucaut Sinusoidal")
 PROJ_HEAD(gall, "Gall (Gall Stereographic)")
+PROJ_HEAD(geocent, "Geocentric")
 PROJ_HEAD(gins8, "Ginsburg VIII (TsNIIGAiK)")
 PROJ_HEAD(gn_sinu, "General Sinusoidal Series")
 PROJ_HEAD(gnom, "Gnomonic")
@@ -47,6 +48,7 @@ PROJ_HEAD(hatano, "Hatano Asymmetrical Equal Area")
 PROJ_HEAD(imw_p, "Internation Map of the World Polyconic")
 PROJ_HEAD(kav5, "Kavraisky V")
 PROJ_HEAD(kav7, "Kavraisky VII")
+PROJ_HEAD(krovak, "Krovak")
 PROJ_HEAD(labrd, "Laborde")
 PROJ_HEAD(laea, "Lambert Azimuthal Equal Area")
 PROJ_HEAD(lagrng, "Lagrange")
@@ -55,6 +57,7 @@ PROJ_HEAD(lask, "Laskowski")
 PROJ_HEAD(latlong, "Lat/long (Geodetic)")
 PROJ_HEAD(longlat, "Lat/long (Geodetic)")
 PROJ_HEAD(lcc, "Lambert Conformal Conic")
+PROJ_HEAD(lcca, "Lambert Conformal Conic Alternative")
 PROJ_HEAD(leac, "Lambert Equal Area Conic")
 PROJ_HEAD(lee_os, "Lee Oblated Stereographic")
 PROJ_HEAD(loxim, "Loximuthal")
